# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ledsettings.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11ledsettings.proto\x12\ndevice.api\"i\n\x0eLedSettingsGet\x12\x32\n\x05state\x18\x01 \x01(\x0e\x32#.device.api.LedSettingsGet.Ledstate\"#\n\x08Ledstate\x12\n\n\x06LED_ON\x10\x00\x12\x0b\n\x07LED_OFF\x10\x01\"i\n\x0eLedSettingsSet\x12\x32\n\x05state\x18\x01 \x01(\x0e\x32#.device.api.LedSettingsSet.Ledstate\"#\n\x08Ledstate\x12\n\n\x06LED_ON\x10\x00\x12\x0b\n\x07LED_OFF\x10\x01\"~\n\x16LedSettingsSetResponse\x12\x39\n\x06result\x18\x01 \x01(\x0e\x32).device.api.LedSettingsSetResponse.Result\")\n\x06Result\x12\x0b\n\x07SUCCESS\x10\x00\x12\x12\n\rUNKNOWN_ERROR\x10\xff\x01\x42\x0e\n\x06\x64\x65viceB\x04ledsb\x06proto3')



_LEDSETTINGSGET = DESCRIPTOR.message_types_by_name['LedSettingsGet']
_LEDSETTINGSSET = DESCRIPTOR.message_types_by_name['LedSettingsSet']
_LEDSETTINGSSETRESPONSE = DESCRIPTOR.message_types_by_name['LedSettingsSetResponse']
_LEDSETTINGSGET_LEDSTATE = _LEDSETTINGSGET.enum_types_by_name['Ledstate']
_LEDSETTINGSSET_LEDSTATE = _LEDSETTINGSSET.enum_types_by_name['Ledstate']
_LEDSETTINGSSETRESPONSE_RESULT = _LEDSETTINGSSETRESPONSE.enum_types_by_name['Result']
LedSettingsGet = _reflection.GeneratedProtocolMessageType('LedSettingsGet', (_message.Message,), {
  'DESCRIPTOR' : _LEDSETTINGSGET,
  '__module__' : 'ledsettings_pb2'
  # @@protoc_insertion_point(class_scope:device.api.LedSettingsGet)
  })
_sym_db.RegisterMessage(LedSettingsGet)

LedSettingsSet = _reflection.GeneratedProtocolMessageType('LedSettingsSet', (_message.Message,), {
  'DESCRIPTOR' : _LEDSETTINGSSET,
  '__module__' : 'ledsettings_pb2'
  # @@protoc_insertion_point(class_scope:device.api.LedSettingsSet)
  })
_sym_db.RegisterMessage(LedSettingsSet)

LedSettingsSetResponse = _reflection.GeneratedProtocolMessageType('LedSettingsSetResponse', (_message.Message,), {
  'DESCRIPTOR' : _LEDSETTINGSSETRESPONSE,
  '__module__' : 'ledsettings_pb2'
  # @@protoc_insertion_point(class_scope:device.api.LedSettingsSetResponse)
  })
_sym_db.RegisterMessage(LedSettingsSetResponse)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\006deviceB\004leds'
  _LEDSETTINGSGET._serialized_start=33
  _LEDSETTINGSGET._serialized_end=138
  _LEDSETTINGSGET_LEDSTATE._serialized_start=103
  _LEDSETTINGSGET_LEDSTATE._serialized_end=138
  _LEDSETTINGSSET._serialized_start=140
  _LEDSETTINGSSET._serialized_end=245
  _LEDSETTINGSSET_LEDSTATE._serialized_start=103
  _LEDSETTINGSSET_LEDSTATE._serialized_end=138
  _LEDSETTINGSSETRESPONSE._serialized_start=247
  _LEDSETTINGSSETRESPONSE._serialized_end=373
  _LEDSETTINGSSETRESPONSE_RESULT._serialized_start=332
  _LEDSETTINGSSETRESPONSE_RESULT._serialized_end=373
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class SupportInfoDump(google.protobuf.message.Message):
    """
    The SupportInfoDump is a collection of SupportInfoItems.
    Each SupportInfoItem is a pair of a unique label and any content.
    The label describes the type of support info that is contained in
    the content field. It can be used by the client to form a file name
    for a file that the content is stored in.
    Labels must be unique within the collection of SupportInfoItems.
    The content can be any data that should be delivered to support for
    the given label and is mostly opaque to the client.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class SupportInfoItem(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        LABEL_FIELD_NUMBER: builtins.int
        CONTENT_FIELD_NUMBER: builtins.int
        label: builtins.str
        """Label describing the content. Defined labels: configlayer, delosconfig, pib, syslog"""
        content: builtins.bytes
        """Any type of support info content."""
        def __init__(
            self,
            *,
            label: builtins.str = ...,
            content: builtins.bytes = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["content", b"content", "label", b"label"]) -> None: ...

    ITEMS_FIELD_NUMBER: builtins.int
    @property
    def items(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___SupportInfoDump.SupportInfoItem]:
        """A list of label-content pairs that make up multiple support info items for this device."""
    def __init__(
        self,
        *,
        items: collections.abc.Iterable[global___SupportInfoDump.SupportInfoItem] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["items", b"items"]) -> None: ...

global___SupportInfoDump = SupportInfoDump

class SupportInfoDumpResponse(google.protobuf.message.Message):
    """
    Response to calling the "SupportInfoDump" endpoint to collect support info data from a device.
    The response is made up of a result code, in case something went wrong, and a SupportInfoDump
    which contains the collection of support info items that the client needs to store in a file
    to be sent to devolo support.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Result:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _ResultEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[SupportInfoDumpResponse._Result.ValueType], builtins.type):  # noqa: F821
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        SUCCESS: SupportInfoDumpResponse._Result.ValueType  # 0
        """Successful operation, support items are returned."""
        UNKNOWN_ERROR: SupportInfoDumpResponse._Result.ValueType  # 255
        """Some error occured, returned info is unreliable."""

    class Result(_Result, metaclass=_ResultEnumTypeWrapper): ...
    SUCCESS: SupportInfoDumpResponse.Result.ValueType  # 0
    """Successful operation, support items are returned."""
    UNKNOWN_ERROR: SupportInfoDumpResponse.Result.ValueType  # 255
    """Some error occured, returned info is unreliable."""

    RESULT_FIELD_NUMBER: builtins.int
    INFO_FIELD_NUMBER: builtins.int
    result: global___SupportInfoDumpResponse.Result.ValueType
    """The return code of the opration, success or error."""
    @property
    def info(self) -> global___SupportInfoDump:
        """The collected support information."""
    def __init__(
        self,
        *,
        result: global___SupportInfoDumpResponse.Result.ValueType = ...,
        info: global___SupportInfoDump | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["info", b"info"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["info", b"info", "result", b"result"]) -> None: ...

global___SupportInfoDumpResponse = SupportInfoDumpResponse



"""Implementation of the devolo device API."""
from __future__ import annotations

import functools
from typing import TYPE_CHECKING, Callable, TypeVar

from devolo_plc_api.clients import Protobuf
from devolo_plc_api.exceptions import FeatureNotSupported

from .factoryreset_pb2 import FactoryResetStart
from .ledsettings_pb2 import LedSettingsGet, LedSettingsSet, LedSettingsSetResponse
from .multiap_pb2 import WifiMultiApGetResponse
from .restart_pb2 import RestartResponse, UptimeGetResponse
from .support_pb2 import SupportInfoDump, SupportInfoDumpResponse
from .updatefirmware_pb2 import UpdateFirmwareCheck, UpdateFirmwareStart
from .wifinetwork_pb2 import (
    WifiConnectedStationsGet,
    WifiGuestAccessGet,
    WifiGuestAccessSet,
    WifiGuestAccessSetResponse,
    WifiNeighborAPsGet,
    WifiRepeatedAPsGet,
    WifiRepeaterWpsClonePbcStart,
    WifiResult,
    WifiWpsPbcStart,
)

if TYPE_CHECKING:
    from httpx import AsyncClient
    from typing_extensions import Concatenate, ParamSpec

    from devolo_plc_api.zeroconf import ZeroconfServiceInfo

    _ReturnT = TypeVar("_ReturnT")
    _P = ParamSpec("_P")


def _feature(
    feature: str,
) -> Callable[[Callable[Concatenate[DeviceApi, _P], _ReturnT]], Callable[Concatenate[DeviceApi, _P], _ReturnT]]:
    """Filter unsupported features before querying the device."""

    def feature_decorator(method: Callable[Concatenate[DeviceApi, _P], _ReturnT]) -> Callable[..., _ReturnT]:
        @functools.wraps(method)
        def wrapper(deviceapi: DeviceApi, *args: _P.args, **kwargs: _P.kwargs) -> _ReturnT:
            if feature in deviceapi.features:
                return method(deviceapi, *args, **kwargs)
            raise FeatureNotSupported(method.__name__)

        return wrapper

    return feature_decorator


class DeviceApi(Protobuf):
    """
    Implementation of the devolo device API.

    :param ip: IP address of the device to communicate with
    :param session: HTTP client session
    :param info: Information collected from the mDNS query
    """

    def __init__(self, ip: str, session: AsyncClient, info: ZeroconfServiceInfo) -> None:
        """Initialize the device API."""
        super().__init__()

        self._ip = ip
        # HC gateway has no Path, it has a path.
        self._path = info.properties.get("Path") or info.properties.get("path")
        self._port = info.port
        self._session = session
        self._user = "devolo"
        self._version = info.properties["Version"]

        features: str = info.properties.get("Features", "")
        self.features = features.split(",") if features else ["reset", "update", "led", "intmtg"]
        self.password = ""

    @_feature("led")
    async def async_get_led_setting(self) -> bool:
        """
        Get LED setting asynchronously. This feature only works on devices, that announce the led feature.

        return: LED settings
        """
        self._logger.debug("Getting LED settings.")
        led_setting = LedSettingsGet()
        response = await self._async_get("LedSettingsGet")
        led_setting.ParseFromString(await response.aread())
        return led_setting.state == led_setting.LED_ON

    @_feature("led")
    async def async_set_led_setting(self, enable: bool) -> bool:
        """
        Set LED setting asynchronously. This feature only works on devices, that announce the led feature.

        :param enable: True to enable the LEDs, False to disable the LEDs
        :return: True, if LED state was successfully changed, otherwise False
        """
        self._logger.debug("Setting LED settings.")
        led_setting = LedSettingsSet()
        led_setting.state = led_setting.LED_ON if enable else led_setting.LED_OFF
        query = await self._async_post("LedSettingsSet", content=led_setting.SerializeToString())
        response = LedSettingsSetResponse()
        response.ParseFromString(await query.aread())
        return response.result == response.SUCCESS

    @_feature("multiap")
    async def async_get_wifi_multi_ap(self) -> WifiMultiApGetResponse:
        """
        Get MultiAP details asynchronously. This feature only works on devices, that announce the multiap feature.

        return: MultiAP details
        """
        self._logger.debug("Getting MultiAP details.")
        query = await self._async_get("WifiMultiApGet")
        response = WifiMultiApGetResponse()
        response.ParseFromString(await query.aread())
        return response

    @_feature("repeater0")
    async def async_get_wifi_repeated_access_points(self) -> list[WifiRepeatedAPsGet.RepeatedAPInfo]:
        """
        Get repeated wifi access point asynchronously. This feature only works on repeater devices, that announce the
        repeater0 feature.

        :return: Repeated access points in the neighborhood including connection rate data
        """
        self._logger.debug("Getting repeated access points.")
        repeated_aps = WifiRepeatedAPsGet()
        response = await self._async_get("WifiRepeatedAPsGet")
        repeated_aps.ParseFromString(await response.aread())
        return list(repeated_aps.repeated_aps)

    @_feature("repeater0")
    async def async_start_wps_clone(self) -> bool:
        """
        Start WPS clone mode. This feature only works on repeater devices, that announce the repeater0 feature.

        :return: True, if the wifi settings were successfully cloned, otherwise False
        """
        self._logger.debug("Starting WPS clone.")
        wps_clone = WifiRepeaterWpsClonePbcStart()
        response = await self._async_get("WifiRepeaterWpsClonePbcStart")
        wps_clone.ParseFromString(await response.aread())
        return wps_clone.result == WifiResult.WIFI_SUCCESS

    @_feature("reset")
    async def async_factory_reset(self) -> bool:
        """
        Factory-reset the device. This feature only works on devices, that announce the reset feature.

        :return: True if reset is started, otherwise False
        """
        self._logger.debug("Resetting the device.")
        reset = FactoryResetStart()
        response = await self._async_get("FactoryResetStart")
        reset.ParseFromString(await response.aread())
        return reset.result == reset.SUCCESS

    @_feature("restart")
    async def async_restart(self) -> bool:
        """
        Restart the device. This feature only works on devices, that announce the restart feature.

        :return: True if restart is started, otherwise False
        """
        self._logger.debug("Restarting the device.")
        query = await self._async_post("Restart", content=b"")
        response = RestartResponse()
        response.ParseFromString(await query.aread())
        return response.result == response.SUCCESS

    @_feature("restart")
    async def async_uptime(self) -> int:
        """
        Get the uptime of the device. This feature only works on devices, that announce the restart feature. It can only be
        used as a strict monotonically increasing number and therefore has no unit.

        :return: The uptime without unit
        """
        self._logger.debug("Get uptime.")
        uptime = UptimeGetResponse()
        response = await self._async_get("UptimeGet")
        uptime.ParseFromString(await response.aread())
        return uptime.uptime

    @_feature("support")
    async def async_get_support_info(self) -> SupportInfoDump:
        """
        Get support info from the device. This feature only works on devices, that announce the support feature.

        :return: The support info
        """
        self._logger.debug("Get uptime.")
        support_info = SupportInfoDumpResponse()
        response = await self._async_get("SupportInfoDump")
        support_info.ParseFromString(await response.aread())
        return support_info.info

    @_feature("update")
    async def async_check_firmware_available(self) -> UpdateFirmwareCheck:
        """
        Check asynchronously, if a firmware update is available for the device.

        :return: Result and new firmware version, if newer one is available
        """
        self._logger.debug("Checking for new firmware.")
        update_firmware_check = UpdateFirmwareCheck()
        response = await self._async_get("UpdateFirmwareCheck", timeout=30.0)
        update_firmware_check.ParseFromString(await response.aread())
        return update_firmware_check

    @_feature("update")
    async def async_start_firmware_update(self) -> bool:
        """
        Start firmware update asynchronously, if a firmware update is available for the device. Important: The response does
        not tell you anything about the success of the update itself.

        :return: True, if the firmware update was started, False if there is no update
        """
        self._logger.debug("Updating firmware.")
        update_firmware = UpdateFirmwareStart()
        query = await self._async_get("UpdateFirmwareStart")
        update_firmware.ParseFromString(await query.aread())
        return update_firmware.result == update_firmware.UPDATE_STARTED

    @_feature("wifi1")
    async def async_get_wifi_connected_station(self) -> list[WifiConnectedStationsGet.ConnectedStationInfo]:
        """
        Get wifi stations connected to the device asynchronously. This feature only works on devices, that announce the wifi1
        feature.

        :return: All connected wifi stations including connection rate data
        """
        self._logger.debug("Getting connected wifi stations.")
        wifi_connected = WifiConnectedStationsGet()
        response = await self._async_get("WifiConnectedStationsGet")
        wifi_connected.ParseFromString(await response.aread())
        return list(wifi_connected.connected_stations)

    @_feature("wifi1")
    async def async_get_wifi_guest_access(self) -> WifiGuestAccessGet:
        """
        Get details about wifi guest access asynchronously. This feature only works on devices, that announce the wifi1
        feature.

        :return: Details about the wifi guest access
        """
        self._logger.debug("Getting wifi guest access status.")
        wifi_guest = WifiGuestAccessGet()
        response = await self._async_get("WifiGuestAccessGet")
        wifi_guest.ParseFromString(await response.aread())
        return wifi_guest

    @_feature("wifi1")
    async def async_set_wifi_guest_access(self, enable: bool, duration: int = 0) -> bool:
        """
        Enable wifi guest access asynchronously. This feature only works on devices, that announce the wifi1 feature.

        :param enable: True to enable, False to disable wifi guest access
        :param duration: Duration in minutes to enable the guest wifi. 0 is infinite.
        :return: True, if the state of the wifi guest access was successfully changed, otherwise False
        """
        self._logger.debug("Setting wifi guest access status.")
        wifi_guest = WifiGuestAccessSet()
        wifi_guest.enable = enable
        wifi_guest.duration = duration
        query = await self._async_post("WifiGuestAccessSet", content=wifi_guest.SerializeToString())
        response = WifiGuestAccessSetResponse()
        response.ParseFromString(await query.aread())
        return response.result == WifiResult.WIFI_SUCCESS

    @_feature("wifi1")
    async def async_get_wifi_neighbor_access_points(self) -> list[WifiNeighborAPsGet.NeighborAPInfo]:
        """
        Get wifi access point in the neighborhood asynchronously. This feature only works on devices, that announce the wifi1
        feature.

        :return: Visible access points in the neighborhood including connection rate data
        """
        self._logger.debug("Getting neighbored access points.")
        wifi_neighbor_aps = WifiNeighborAPsGet()
        response = await self._async_get("WifiNeighborAPsGet", timeout=30.0)
        wifi_neighbor_aps.ParseFromString(await response.aread())
        return list(wifi_neighbor_aps.neighbor_aps)

    @_feature("wifi1")
    async def async_start_wps(self) -> bool:
        """
        Start WPS push button configuration.

        :return: True, if the WPS was successfully started, otherwise False
        """
        self._logger.debug("Starting WPS.")
        wps = WifiWpsPbcStart()
        response = await self._async_get("WifiWpsPbcStart")
        wps.ParseFromString(await response.aread())
        return wps.result == WifiResult.WIFI_SUCCESS



"""Fixtures for plcnet API tests."""
from typing import AsyncGenerator

import pytest
import pytest_asyncio
from httpx import AsyncClient

from devolo_plc_api.plcnet_api import SERVICE_TYPE, LogicalNetwork, PlcNetApi

from tests import TestData


@pytest_asyncio.fixture()
async def plcnet_api(test_data: TestData) -> AsyncGenerator[PlcNetApi, None]:
    """Yield a prepared PlcNetApi object."""
    async with AsyncClient() as client:
        yield PlcNetApi(test_data.ip, client, test_data.device_info[SERVICE_TYPE])


@pytest.fixture()
def network() -> LogicalNetwork:
    """Mock a PLC network."""
    return LogicalNetwork(devices=[], data_rates=[])



"""Exceptions used by the package."""
from .device import DeviceNotFound, DevicePasswordProtected, DeviceUnavailable
from .feature import FeatureNotSupported

__all__ = ["DeviceNotFound", "DevicePasswordProtected", "DeviceUnavailable", "FeatureNotSupported"]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: factoryreset.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12\x66\x61\x63toryreset.proto\x12\ndevice.api\"t\n\x11\x46\x61\x63toryResetStart\x12\x34\n\x06result\x18\x01 \x01(\x0e\x32$.device.api.FactoryResetStart.Result\")\n\x06Result\x12\x0b\n\x07SUCCESS\x10\x00\x12\x12\n\rUNKNOWN_ERROR\x10\xff\x01\x42\x16\n\x06\x64\x65viceB\x0c\x66\x61\x63toryresetb\x06proto3')



_FACTORYRESETSTART = DESCRIPTOR.message_types_by_name['FactoryResetStart']
_FACTORYRESETSTART_RESULT = _FACTORYRESETSTART.enum_types_by_name['Result']
FactoryResetStart = _reflection.GeneratedProtocolMessageType('FactoryResetStart', (_message.Message,), {
  'DESCRIPTOR' : _FACTORYRESETSTART,
  '__module__' : 'factoryreset_pb2'
  # @@protoc_insertion_point(class_scope:device.api.FactoryResetStart)
  })
_sym_db.RegisterMessage(FactoryResetStart)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\006deviceB\014factoryreset'
  _FACTORYRESETSTART._serialized_start=34
  _FACTORYRESETSTART._serialized_end=150
  _FACTORYRESETSTART_RESULT._serialized_start=109
  _FACTORYRESETSTART_RESULT._serialized_end=150
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class SetUserDeviceName(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    MAC_ADDRESS_FIELD_NUMBER: builtins.int
    USER_DEVICE_NAME_FIELD_NUMBER: builtins.int
    mac_address: typing.Text
    """MAC address of the targeted device"""

    user_device_name: typing.Text
    """user provided device name, if any"""

    def __init__(self,
        *,
        mac_address: typing.Text = ...,
        user_device_name: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["mac_address",b"mac_address","user_device_name",b"user_device_name"]) -> None: ...
global___SetUserDeviceName = SetUserDeviceName

class SetUserDeviceNameResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _Result:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _ResultEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[SetUserDeviceNameResponse._Result.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        SUCCESS: SetUserDeviceNameResponse._Result.ValueType  # 0
        MACADDR_INVALID: SetUserDeviceNameResponse._Result.ValueType  # 1
        MACADDR_UNKNOWN: SetUserDeviceNameResponse._Result.ValueType  # 2
        DEVICE_NAME_INVALID: SetUserDeviceNameResponse._Result.ValueType  # 3
        COMMUNICATION_ERROR: SetUserDeviceNameResponse._Result.ValueType  # 254
        UNKNOWN_ERROR: SetUserDeviceNameResponse._Result.ValueType  # 255
    class Result(_Result, metaclass=_ResultEnumTypeWrapper):
        pass

    SUCCESS: SetUserDeviceNameResponse.Result.ValueType  # 0
    MACADDR_INVALID: SetUserDeviceNameResponse.Result.ValueType  # 1
    MACADDR_UNKNOWN: SetUserDeviceNameResponse.Result.ValueType  # 2
    DEVICE_NAME_INVALID: SetUserDeviceNameResponse.Result.ValueType  # 3
    COMMUNICATION_ERROR: SetUserDeviceNameResponse.Result.ValueType  # 254
    UNKNOWN_ERROR: SetUserDeviceNameResponse.Result.ValueType  # 255

    RESULT_FIELD_NUMBER: builtins.int
    result: global___SetUserDeviceNameResponse.Result.ValueType
    """contains the result of SetUserDeviceName message"""

    def __init__(self,
        *,
        result: global___SetUserDeviceNameResponse.Result.ValueType = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["result",b"result"]) -> None: ...
global___SetUserDeviceNameResponse = SetUserDeviceNameResponse



"""Test communicating with a devolo device."""
from asyncio import AbstractEventLoop
from unittest.mock import AsyncMock, Mock, patch

import pytest
from syrupy.assertion import SnapshotAssertion
from zeroconf import ServiceStateChange

from devolo_plc_api.device import Device
from devolo_plc_api.exceptions import DeviceNotFound
from devolo_plc_api.plcnet_api import SERVICE_TYPE as PLCNETAPI

from . import DeviceType, TestData
from .mocks.zeroconf import MockAsyncServiceInfo


@pytest.mark.usefixtures("block_communication")
class TestDevice:
    """Test devolo_plc_api.device.Device class."""

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("device_type", [DeviceType.PLC])
    @pytest.mark.usefixtures("service_browser")
    async def test_async_connect_plc(self, mock_device: Device, snapshot: SnapshotAssertion):
        """Test that connecting to a device collects information from the APIs."""
        await mock_device.async_connect()
        assert mock_device._connected
        assert mock_device.device
        assert mock_device.plcnet
        assert mock_device == snapshot
        await mock_device.async_disconnect()

        await mock_device.async_connect(session_instance=AsyncMock())
        assert mock_device._connected
        assert mock_device.device
        assert mock_device.plcnet
        assert mock_device == snapshot
        await mock_device.async_disconnect()

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("device_type", [DeviceType.REPEATER])
    @pytest.mark.usefixtures("service_browser")
    async def test_async_connect_repeater(self, mock_device: Device, snapshot: SnapshotAssertion):
        """Test that connecting to a device collects information from the APIs."""
        await mock_device.async_connect()
        assert mock_device._connected
        assert mock_device.device
        assert not mock_device.plcnet
        assert mock_device == snapshot
        await mock_device.async_disconnect()

    @pytest.mark.asyncio()
    async def test_sync_connect_multicast(self, test_data: TestData):
        """Test that devices having trouble with unicast zeroconf are queried twice."""
        with patch("devolo_plc_api.device.Device._get_zeroconf_info") as get_zeroconf_info, pytest.raises(DeviceNotFound):
            device = Device(test_data.ip)
            await device.async_connect()
            assert device._multicast
            assert get_zeroconf_info.call_count == 2

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("device_type", [DeviceType.PLC])
    @pytest.mark.usefixtures("service_browser")
    async def test_async_connect_not_found(self, mock_device: Device, sleep: AsyncMock):
        """Test that an exception is raised if both APIs are not available."""
        with pytest.raises(DeviceNotFound):
            await mock_device.async_connect()
            assert not mock_device._connected
            assert sleep.call_count == Device.MDNS_TIMEOUT

    def test_connect(self, mock_device: Device):
        """Test that the sync connect method just calls the async connect method."""
        with patch("devolo_plc_api.device.Device.async_connect", new=AsyncMock()) as ac:
            mock_device.connect()
            assert ac.call_count == 1

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("device_type", [DeviceType.PLC])
    @pytest.mark.usefixtures("service_browser")
    async def test_set_password(self, mock_device: Device):
        """Test setting a device password is also reflected in the APIs."""
        await mock_device.async_connect()
        assert mock_device.device
        assert mock_device.plcnet
        mock_device.password = "super_secret"
        assert mock_device.device.password == "super_secret"
        assert mock_device.plcnet.password == "super_secret"

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("device_type", [DeviceType.PLC])
    @pytest.mark.usefixtures("http_client", "service_browser")
    async def test_async_disconnect(self, mock_device: Device):
        """Test that disconnecting from a device cleans up Zeroconf and the HTTP client."""
        await mock_device.async_connect()
        await mock_device.async_disconnect()
        assert mock_device._zeroconf.async_close.call_count == 1  # type: ignore[attr-defined]
        assert mock_device._session.aclose.call_count == 1  # type: ignore[attr-defined]
        assert not mock_device._connected

        await mock_device.async_connect(session_instance=AsyncMock())
        await mock_device.async_disconnect()
        assert mock_device._zeroconf.async_close.call_count == 1  # type: ignore[attr-defined]
        assert mock_device._session.aclose.call_count == 0  # type: ignore[attr-defined]
        assert not mock_device._connected

    def test_disconnect(self, mock_device: Device, event_loop: AbstractEventLoop):
        """Test that the sync disconnect method just calls the async disconnect method."""
        with patch("devolo_plc_api.device.Device.async_disconnect", new=AsyncMock()) as ad:
            mock_device._loop = event_loop
            mock_device.disconnect()
            assert ad.call_count == 1

    @pytest.mark.asyncio()
    @pytest.mark.parametrize("device_type", [DeviceType.PLC])
    @pytest.mark.usefixtures("service_browser")
    async def test_async_context_manager(self, test_data: TestData):
        """Test the async context manager."""
        async with Device(test_data.ip) as device:
            assert device._connected
        assert not device._connected

    @pytest.mark.usefixtures("service_browser")
    @pytest.mark.parametrize("device_type", [DeviceType.PLC])
    def test_context_manager(self, test_data: TestData):
        """Test the sync context manager."""
        with Device(test_data.ip) as device:
            assert device._connected
        assert not device._connected

    @pytest.mark.asyncio()
    async def test_state_change_removed(self, mock_device: Device):
        """Test that service information are not processed on state change to removed."""
        with patch("devolo_plc_api.device.Device._retry_zeroconf_info"), patch(
            "devolo_plc_api.device.Device._get_service_info"
        ) as gsi:
            mock_device._state_change(Mock(), PLCNETAPI, PLCNETAPI, ServiceStateChange.Removed)
            assert gsi.call_count == 0

    @pytest.mark.asyncio()
    @pytest.mark.usefixtures("service_browser", "sleep")
    @pytest.mark.parametrize("device_type", [DeviceType.PLC])
    async def test_get_service_info_alien(self, mock_info_from_service: Mock):
        """Test ignoring alien information discovered via mDNS."""
        with pytest.raises(DeviceNotFound):
            mock_device = Device(ip="192.0.2.2")
            await mock_device.async_connect()
            assert MockAsyncServiceInfo.async_request.call_count == 1
            assert mock_info_from_service.call_count == 0



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _WifiResult:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _WifiResultEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_WifiResult.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    WIFI_SUCCESS: _WifiResult.ValueType  # 0
    WIFI_INVALID_SSID: _WifiResult.ValueType  # 1
    WIFI_INVALID_KEY: _WifiResult.ValueType  # 2
    WIFI_IS_DISABLED: _WifiResult.ValueType  # 3
    WIFI_INVALID_BACKHAUL_SSID: _WifiResult.ValueType  # 4
    """@since 1"""
    WIFI_INVALID_BACKHAUL_KEY: _WifiResult.ValueType  # 5
    """@since 1"""
    WIFI_UNSUPPORTED_DEVICE_MODE: _WifiResult.ValueType  # 6
    """@since 1"""
    WIFI_UNKNOWN_ERROR: _WifiResult.ValueType  # 255

class WifiResult(_WifiResult, metaclass=_WifiResultEnumTypeWrapper): ...

WIFI_SUCCESS: WifiResult.ValueType  # 0
WIFI_INVALID_SSID: WifiResult.ValueType  # 1
WIFI_INVALID_KEY: WifiResult.ValueType  # 2
WIFI_IS_DISABLED: WifiResult.ValueType  # 3
WIFI_INVALID_BACKHAUL_SSID: WifiResult.ValueType  # 4
"""@since 1"""
WIFI_INVALID_BACKHAUL_KEY: WifiResult.ValueType  # 5
"""@since 1"""
WIFI_UNSUPPORTED_DEVICE_MODE: WifiResult.ValueType  # 6
"""@since 1"""
WIFI_UNKNOWN_ERROR: WifiResult.ValueType  # 255
global___WifiResult = WifiResult

class _WifiBand:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _WifiBandEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_WifiBand.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    WIFI_BAND_UNKNOWN: _WifiBand.ValueType  # 0
    WIFI_BAND_2G: _WifiBand.ValueType  # 1
    WIFI_BAND_5G: _WifiBand.ValueType  # 2

class WifiBand(_WifiBand, metaclass=_WifiBandEnumTypeWrapper): ...

WIFI_BAND_UNKNOWN: WifiBand.ValueType  # 0
WIFI_BAND_2G: WifiBand.ValueType  # 1
WIFI_BAND_5G: WifiBand.ValueType  # 2
global___WifiBand = WifiBand

class _WifiVAPType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _WifiVAPTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_WifiVAPType.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    WIFI_VAP_UNKNOWN: _WifiVAPType.ValueType  # 0
    WIFI_VAP_MAIN_AP: _WifiVAPType.ValueType  # 1
    WIFI_VAP_GUEST_AP: _WifiVAPType.ValueType  # 2
    WIFI_VAP_STATION: _WifiVAPType.ValueType  # 3

class WifiVAPType(_WifiVAPType, metaclass=_WifiVAPTypeEnumTypeWrapper): ...

WIFI_VAP_UNKNOWN: WifiVAPType.ValueType  # 0
WIFI_VAP_MAIN_AP: WifiVAPType.ValueType  # 1
WIFI_VAP_GUEST_AP: WifiVAPType.ValueType  # 2
WIFI_VAP_STATION: WifiVAPType.ValueType  # 3
global___WifiVAPType = WifiVAPType

class _WifiWpaMode:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _WifiWpaModeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_WifiWpaMode.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    WPA_NO_CHANGE: _WifiWpaMode.ValueType  # 0
    WPA_NONE: _WifiWpaMode.ValueType  # 1
    WPA_1_2: _WifiWpaMode.ValueType  # 2
    WPA_2: _WifiWpaMode.ValueType  # 3
    WPA_2_3: _WifiWpaMode.ValueType  # 4
    WPA_3: _WifiWpaMode.ValueType  # 5

class WifiWpaMode(_WifiWpaMode, metaclass=_WifiWpaModeEnumTypeWrapper):
    """@since 1"""

WPA_NO_CHANGE: WifiWpaMode.ValueType  # 0
WPA_NONE: WifiWpaMode.ValueType  # 1
WPA_1_2: WifiWpaMode.ValueType  # 2
WPA_2: WifiWpaMode.ValueType  # 3
WPA_2_3: WifiWpaMode.ValueType  # 4
WPA_3: WifiWpaMode.ValueType  # 5
global___WifiWpaMode = WifiWpaMode

class _WifiDeviceMode:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _WifiDeviceModeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_WifiDeviceMode.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    WDM_NO_CONF: _WifiDeviceMode.ValueType  # 0
    """Unconfigured, built in automation defines operating mode"""
    WDM_AP: _WifiDeviceMode.ValueType  # 2
    """Configured to run as access point"""
    WDM_REPEATER: _WifiDeviceMode.ValueType  # 3
    """Configured to run as WiFi repeater"""

class WifiDeviceMode(_WifiDeviceMode, metaclass=_WifiDeviceModeEnumTypeWrapper):
    """@since 1"""

WDM_NO_CONF: WifiDeviceMode.ValueType  # 0
"""Unconfigured, built in automation defines operating mode"""
WDM_AP: WifiDeviceMode.ValueType  # 2
"""Configured to run as access point"""
WDM_REPEATER: WifiDeviceMode.ValueType  # 3
"""Configured to run as WiFi repeater"""
global___WifiDeviceMode = WifiDeviceMode

class WifiDeviceModeGetResponse(google.protobuf.message.Message):
    """@since 1"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MODE_FIELD_NUMBER: builtins.int
    mode: global___WifiDeviceMode.ValueType
    """The mode the device is currently operating in, e.g. repeater, AP."""
    def __init__(
        self,
        *,
        mode: global___WifiDeviceMode.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["mode", b"mode"]) -> None: ...

global___WifiDeviceModeGetResponse = WifiDeviceModeGetResponse

class WifiDeviceModeSet(google.protobuf.message.Message):
    """@since 1"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MODE_FIELD_NUMBER: builtins.int
    FH_PARAMS_FIELD_NUMBER: builtins.int
    BH_PARAMS_FIELD_NUMBER: builtins.int
    mode: global___WifiDeviceMode.ValueType
    """The mode the device shall operate in, e.g. repeater, AP."""
    @property
    def fh_params(self) -> global___WifiParametersSet:
        """The front haul WiFi parameters, i.e. the WiFi the clients connect to."""
    @property
    def bh_params(self) -> global___WifiRepeaterParametersSet:
        """  When left out, parameters are not changed on the device.
        The back haul WiFi parameters, i.e. the gateway's WiFi the repeater connects to.
        """
    def __init__(
        self,
        *,
        mode: global___WifiDeviceMode.ValueType = ...,
        fh_params: global___WifiParametersSet | None = ...,
        bh_params: global___WifiRepeaterParametersSet | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["bh_params", b"bh_params", "fh_params", b"fh_params"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["bh_params", b"bh_params", "fh_params", b"fh_params", "mode", b"mode"]) -> None: ...

global___WifiDeviceModeSet = WifiDeviceModeSet

class WifiDeviceModeSetResponse(google.protobuf.message.Message):
    """@since 1"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESULT_FIELD_NUMBER: builtins.int
    result: global___WifiResult.ValueType
    """Success or failure status."""
    def __init__(
        self,
        *,
        result: global___WifiResult.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["result", b"result"]) -> None: ...

global___WifiDeviceModeSetResponse = WifiDeviceModeSetResponse

class WifiParametersSet(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SSID_FIELD_NUMBER: builtins.int
    KEY_FIELD_NUMBER: builtins.int
    ssid: builtins.str
    """WiFi ssid"""
    key: builtins.str
    """WiFi key, independent of the type (WPA2, ...)"""
    def __init__(
        self,
        *,
        ssid: builtins.str = ...,
        key: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "ssid", b"ssid"]) -> None: ...

global___WifiParametersSet = WifiParametersSet

class WifiParametersSetResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESULT_FIELD_NUMBER: builtins.int
    result: global___WifiResult.ValueType
    """contains the result of setting WiFi Parameter message"""
    def __init__(
        self,
        *,
        result: global___WifiResult.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["result", b"result"]) -> None: ...

global___WifiParametersSetResponse = WifiParametersSetResponse

class WifiGuestAccessSet(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ENABLE_FIELD_NUMBER: builtins.int
    DURATION_FIELD_NUMBER: builtins.int
    SSID_FIELD_NUMBER: builtins.int
    KEY_FIELD_NUMBER: builtins.int
    WPA_FIELD_NUMBER: builtins.int
    enable: builtins.bool
    duration: builtins.int
    """time in minutes; 0 is infinity"""
    ssid: builtins.str
    """Guest WiFi SSID. Leaving it out means not to change the current value.  (@since 1)"""
    key: builtins.str
    """Guest WiFi key. Leaving it out means not to change the current value.   (@since 1)"""
    wpa: global___WifiWpaMode.ValueType
    """Encryption mode. Leaving it out means not to change the current value.  (@since 1)"""
    def __init__(
        self,
        *,
        enable: builtins.bool = ...,
        duration: builtins.int = ...,
        ssid: builtins.str = ...,
        key: builtins.str = ...,
        wpa: global___WifiWpaMode.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["duration", b"duration", "enable", b"enable", "key", b"key", "ssid", b"ssid", "wpa", b"wpa"]) -> None: ...

global___WifiGuestAccessSet = WifiGuestAccessSet

class WifiGuestAccessSetResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESULT_FIELD_NUMBER: builtins.int
    result: global___WifiResult.ValueType
    """contains the result of setting WiFi Guest Access parameters"""
    def __init__(
        self,
        *,
        result: global___WifiResult.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["result", b"result"]) -> None: ...

global___WifiGuestAccessSetResponse = WifiGuestAccessSetResponse

class WifiGuestAccessGet(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ENABLED_FIELD_NUMBER: builtins.int
    REMAINING_DURATION_FIELD_NUMBER: builtins.int
    SSID_FIELD_NUMBER: builtins.int
    KEY_FIELD_NUMBER: builtins.int
    WPA_FIELD_NUMBER: builtins.int
    enabled: builtins.bool
    remaining_duration: builtins.int
    ssid: builtins.str
    key: builtins.str
    wpa: global___WifiWpaMode.ValueType
    """Encryption mode.  (@since 1)"""
    def __init__(
        self,
        *,
        enabled: builtins.bool = ...,
        remaining_duration: builtins.int = ...,
        ssid: builtins.str = ...,
        key: builtins.str = ...,
        wpa: global___WifiWpaMode.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["enabled", b"enabled", "key", b"key", "remaining_duration", b"remaining_duration", "ssid", b"ssid", "wpa", b"wpa"]) -> None: ...

global___WifiGuestAccessGet = WifiGuestAccessGet

class WifiNeighborAPsGet(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class NeighborAPInfo(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        MAC_ADDRESS_FIELD_NUMBER: builtins.int
        SSID_FIELD_NUMBER: builtins.int
        BAND_FIELD_NUMBER: builtins.int
        CHANNEL_FIELD_NUMBER: builtins.int
        SIGNAL_FIELD_NUMBER: builtins.int
        SIGNAL_BARS_FIELD_NUMBER: builtins.int
        mac_address: builtins.str
        ssid: builtins.str
        band: global___WifiBand.ValueType
        channel: builtins.int
        signal: builtins.int
        signal_bars: builtins.int
        def __init__(
            self,
            *,
            mac_address: builtins.str = ...,
            ssid: builtins.str = ...,
            band: global___WifiBand.ValueType = ...,
            channel: builtins.int = ...,
            signal: builtins.int = ...,
            signal_bars: builtins.int = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["band", b"band", "channel", b"channel", "mac_address", b"mac_address", "signal", b"signal", "signal_bars", b"signal_bars", "ssid", b"ssid"]) -> None: ...

    NEIGHBOR_APS_FIELD_NUMBER: builtins.int
    @property
    def neighbor_aps(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___WifiNeighborAPsGet.NeighborAPInfo]: ...
    def __init__(
        self,
        *,
        neighbor_aps: collections.abc.Iterable[global___WifiNeighborAPsGet.NeighborAPInfo] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["neighbor_aps", b"neighbor_aps"]) -> None: ...

global___WifiNeighborAPsGet = WifiNeighborAPsGet

class WifiRepeatedAPsGet(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class RepeatedAPInfo(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        MAC_ADDRESS_FIELD_NUMBER: builtins.int
        SSID_FIELD_NUMBER: builtins.int
        BAND_FIELD_NUMBER: builtins.int
        CHANNEL_FIELD_NUMBER: builtins.int
        RATE_FIELD_NUMBER: builtins.int
        SIGNAL_FIELD_NUMBER: builtins.int
        SIGNAL_BARS_FIELD_NUMBER: builtins.int
        mac_address: builtins.str
        ssid: builtins.str
        band: global___WifiBand.ValueType
        channel: builtins.int
        rate: builtins.int
        signal: builtins.int
        signal_bars: builtins.int
        def __init__(
            self,
            *,
            mac_address: builtins.str = ...,
            ssid: builtins.str = ...,
            band: global___WifiBand.ValueType = ...,
            channel: builtins.int = ...,
            rate: builtins.int = ...,
            signal: builtins.int = ...,
            signal_bars: builtins.int = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["band", b"band", "channel", b"channel", "mac_address", b"mac_address", "rate", b"rate", "signal", b"signal", "signal_bars", b"signal_bars", "ssid", b"ssid"]) -> None: ...

    REPEATED_APS_FIELD_NUMBER: builtins.int
    @property
    def repeated_aps(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___WifiRepeatedAPsGet.RepeatedAPInfo]: ...
    def __init__(
        self,
        *,
        repeated_aps: collections.abc.Iterable[global___WifiRepeatedAPsGet.RepeatedAPInfo] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["repeated_aps", b"repeated_aps"]) -> None: ...

global___WifiRepeatedAPsGet = WifiRepeatedAPsGet

class WifiConnectedStationsGet(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class ConnectedStationInfo(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        MAC_ADDRESS_FIELD_NUMBER: builtins.int
        VAP_TYPE_FIELD_NUMBER: builtins.int
        BAND_FIELD_NUMBER: builtins.int
        RX_RATE_FIELD_NUMBER: builtins.int
        TX_RATE_FIELD_NUMBER: builtins.int
        mac_address: builtins.str
        vap_type: global___WifiVAPType.ValueType
        band: global___WifiBand.ValueType
        rx_rate: builtins.int
        tx_rate: builtins.int
        def __init__(
            self,
            *,
            mac_address: builtins.str = ...,
            vap_type: global___WifiVAPType.ValueType = ...,
            band: global___WifiBand.ValueType = ...,
            rx_rate: builtins.int = ...,
            tx_rate: builtins.int = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["band", b"band", "mac_address", b"mac_address", "rx_rate", b"rx_rate", "tx_rate", b"tx_rate", "vap_type", b"vap_type"]) -> None: ...

    CONNECTED_STATIONS_FIELD_NUMBER: builtins.int
    @property
    def connected_stations(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___WifiConnectedStationsGet.ConnectedStationInfo]: ...
    def __init__(
        self,
        *,
        connected_stations: collections.abc.Iterable[global___WifiConnectedStationsGet.ConnectedStationInfo] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["connected_stations", b"connected_stations"]) -> None: ...

global___WifiConnectedStationsGet = WifiConnectedStationsGet

class WifiRepeaterParametersSet(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SSID_FIELD_NUMBER: builtins.int
    KEY_FIELD_NUMBER: builtins.int
    CROSSBAND_FIELD_NUMBER: builtins.int
    PRIMARY_BAND_FIELD_NUMBER: builtins.int
    ssid: builtins.str
    """Wifi ssid"""
    key: builtins.str
    """Wifi key, independent of the type (WPA2, ...)"""
    crossband: builtins.bool
    """Use crossband (true) or in-band repeating (false)"""
    primary_band: global___WifiBand.ValueType
    """Primary backhaul band when using crossband"""
    def __init__(
        self,
        *,
        ssid: builtins.str = ...,
        key: builtins.str = ...,
        crossband: builtins.bool = ...,
        primary_band: global___WifiBand.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["crossband", b"crossband", "key", b"key", "primary_band", b"primary_band", "ssid", b"ssid"]) -> None: ...

global___WifiRepeaterParametersSet = WifiRepeaterParametersSet

class WifiRepeaterParametersSetResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESULT_FIELD_NUMBER: builtins.int
    result: global___WifiResult.ValueType
    def __init__(
        self,
        *,
        result: global___WifiResult.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["result", b"result"]) -> None: ...

global___WifiRepeaterParametersSetResponse = WifiRepeaterParametersSetResponse

class WifiWpsPbcStart(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESULT_FIELD_NUMBER: builtins.int
    result: global___WifiResult.ValueType
    def __init__(
        self,
        *,
        result: global___WifiResult.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["result", b"result"]) -> None: ...

global___WifiWpsPbcStart = WifiWpsPbcStart

class WifiRepeaterWpsClonePbcStart(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESULT_FIELD_NUMBER: builtins.int
    result: global___WifiResult.ValueType
    def __init__(
        self,
        *,
        result: global___WifiResult.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["result", b"result"]) -> None: ...

global___WifiRepeaterWpsClonePbcStart = WifiRepeaterWpsClonePbcStart



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class LedSettingsGet(google.protobuf.message.Message):
    """
    Get current LED settings
    Use GET, no payload required
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _Ledstate:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _LedstateEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[LedSettingsGet._Ledstate.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        LED_ON: LedSettingsGet._Ledstate.ValueType  # 0
        LED_OFF: LedSettingsGet._Ledstate.ValueType  # 1
    class Ledstate(_Ledstate, metaclass=_LedstateEnumTypeWrapper):
        pass

    LED_ON: LedSettingsGet.Ledstate.ValueType  # 0
    LED_OFF: LedSettingsGet.Ledstate.ValueType  # 1

    STATE_FIELD_NUMBER: builtins.int
    state: global___LedSettingsGet.Ledstate.ValueType
    def __init__(self,
        *,
        state: global___LedSettingsGet.Ledstate.ValueType = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["state",b"state"]) -> None: ...
global___LedSettingsGet = LedSettingsGet

class LedSettingsSet(google.protobuf.message.Message):
    """
    Set new LED settings
    Use http POST with this message
    Backend will answer with LedSettingsSetResponse
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _Ledstate:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _LedstateEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[LedSettingsSet._Ledstate.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        LED_ON: LedSettingsSet._Ledstate.ValueType  # 0
        LED_OFF: LedSettingsSet._Ledstate.ValueType  # 1
    class Ledstate(_Ledstate, metaclass=_LedstateEnumTypeWrapper):
        pass

    LED_ON: LedSettingsSet.Ledstate.ValueType  # 0
    LED_OFF: LedSettingsSet.Ledstate.ValueType  # 1

    STATE_FIELD_NUMBER: builtins.int
    state: global___LedSettingsSet.Ledstate.ValueType
    def __init__(self,
        *,
        state: global___LedSettingsSet.Ledstate.ValueType = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["state",b"state"]) -> None: ...
global___LedSettingsSet = LedSettingsSet

class LedSettingsSetResponse(google.protobuf.message.Message):
    """
    This will be returned after LedSettingsSet
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _Result:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _ResultEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[LedSettingsSetResponse._Result.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        SUCCESS: LedSettingsSetResponse._Result.ValueType  # 0
        UNKNOWN_ERROR: LedSettingsSetResponse._Result.ValueType  # 255
    class Result(_Result, metaclass=_ResultEnumTypeWrapper):
        pass

    SUCCESS: LedSettingsSetResponse.Result.ValueType  # 0
    UNKNOWN_ERROR: LedSettingsSetResponse.Result.ValueType  # 255

    RESULT_FIELD_NUMBER: builtins.int
    result: global___LedSettingsSetResponse.Result.ValueType
    """contains the result of setting the LED state"""

    def __init__(self,
        *,
        result: global___LedSettingsSetResponse.Result.ValueType = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["result",b"result"]) -> None: ...
global___LedSettingsSetResponse = LedSettingsSetResponse



"""The devolo PLC API."""
from importlib.metadata import PackageNotFoundError, version

from .device import Device
from .helpers import wifi_qr_code

try:
    __version__ = version("devolo_plc_api")
except PackageNotFoundError:
    # package is not installed - e.g. pulled and run locally
    __version__ = "0.0.0"

__all__ = ["Device", "wifi_qr_code", "__version__"]



#!/usr/bin/env python3
"""Generate stub files for API classes with async and sync interface."""
from __future__ import annotations

import re
import sys
from contextlib import suppress
from copy import copy
from pathlib import Path

from mypy.nodes import ConditionalExpr, Expression, ListExpr
from mypy.stubgen import (
    Options,
    StubGenerator,
    StubSource,
    collect_build_targets,
    generate_asts_for_modules,
    generate_guarded,
    mypy_options,
)

HEADER = '''"""
@generated by stubgen.  Do not edit manually!
isort:skip_file
"""
'''


class ApiStubGenerator(StubGenerator):
    """Generate stub text from a mypy AST."""

    def get_str_type_of_node(self, rvalue: Expression, *, can_infer_optional: bool = False, can_be_any: bool = True) -> str:
        """Get type of node as string."""
        if isinstance(rvalue, ConditionalExpr):
            if_type = self.get_str_type_of_node(rvalue.if_expr, can_infer_optional=can_infer_optional, can_be_any=False)
            else_type = self.get_str_type_of_node(rvalue.else_expr, can_infer_optional=can_infer_optional, can_be_any=False)
            if if_type and else_type and if_type != else_type:
                return f"{if_type} | {else_type}"
            return if_type or else_type or "Any" if can_be_any else ""
        if isinstance(rvalue, ListExpr):
            list_item_type = {
                self.get_str_type_of_node(item, can_infer_optional=can_infer_optional, can_be_any=can_be_any)
                for item in rvalue.items
            }
            return f"list[{' | '.join(list_item_type)}]"
        return super().get_str_type_of_node(rvalue, can_infer_optional=can_infer_optional, can_be_any=can_be_any)

    def add_sync(self) -> None:
        """Add sync methods."""
        output = copy(self._output)
        for i in range(len(output)):
            if "async" in output[i]:
                self.add(output[i].replace("async_", "").replace("async ", ""))
                self.add(output[i + 1])
                self.add(output[i + 2])

    def fix_union_annotations(self) -> None:
        """Fix Union annotations."""
        for i, output in enumerate(self._output):
            if match := re.search(r"Union\[([a-z, ]+)\]", output):
                types = match[1].replace(",", " |")
                self._output[i] = output.replace(match[0], types)


def generate_stubs() -> None:
    """Generate stubs - main entry point for the program."""
    options = Options(
        pyversion=sys.version_info[:2],
        no_import=True,
        doc_dir="",
        search_path=[],
        interpreter=sys.executable,
        parse_only=False,
        ignore_errors=False,
        include_private=False,
        output_dir="",
        modules=[],
        packages=[],
        files=["devolo_plc_api/device_api/deviceapi.py", "devolo_plc_api/plcnet_api/plcnetapi.py"],
        verbose=False,
        quiet=True,
        export_less=True,
    )
    mypy_opts = mypy_options(options)
    py_modules, _ = collect_build_targets(options, mypy_opts)
    generate_asts_for_modules(py_modules, options.parse_only, mypy_opts, options.verbose)
    files = []
    for mod in py_modules:
        target = mod.module.replace(".", "/")
        target += ".pyi"
        target = Path(options.output_dir) / target
        files.append(target)
        with generate_guarded(mod.module, target, options.ignore_errors, options.verbose):
            generate_stub_from_ast(mod, target, options.parse_only, options.include_private, options.export_less)


def generate_stub_from_ast(mod: StubSource, target: str, parse_only: bool, include_private: bool, export_less: bool) -> None:
    """Use analyzed (or just parsed) AST to generate type stub for single file."""
    gen = ApiStubGenerator(mod.runtime_all, include_private=include_private, analyzed=not parse_only, export_less=export_less)
    mod.ast.accept(gen)
    if "annotations" in mod.ast.future_import_flags:
        gen.add_import_line("from __future__ import annotations\n")
        gen.fix_union_annotations()
    gen.add_sync()

    old_output = ""
    new_output = HEADER + "".join(gen.output())

    with suppress(FileNotFoundError), Path(target).open() as file:
        old_output = file.read()

    if new_output != old_output:
        with Path(target).open("w") as file:
            file.write(new_output)
            sys.exit(1)


if __name__ == "__main__":
    generate_stubs()



"""Test network discovery."""
from socket import inet_aton
from unittest.mock import Mock, patch

import pytest
from zeroconf import ServiceStateChange, Zeroconf

from devolo_plc_api import Device, network
from devolo_plc_api.device_api import SERVICE_TYPE
from devolo_plc_api.zeroconf import ZeroconfServiceInfo

from . import TestData
from .mocks.zeroconf import MockServiceBrowser


class TestNetwork:
    """Test devolo_plc_api.network functions."""

    @pytest.mark.asyncio()
    async def test_async_discover_network(self, test_data: TestData, mock_info_from_service: Mock):
        """Test discovering the network asynchronously."""
        with patch("devolo_plc_api.network.ServiceBrowser", MockServiceBrowser), patch(
            "devolo_plc_api.network.Zeroconf.get_service_info", return_value=""
        ):
            serial_number = test_data.device_info[SERVICE_TYPE].properties["SN"]
            mock_info_from_service.return_value = ZeroconfServiceInfo(
                address=inet_aton(test_data.ip),
                properties=test_data.device_info[SERVICE_TYPE].properties,
            )
            discovered = await network.async_discover_network(timeout=0.1)
            assert serial_number in discovered
            assert isinstance(discovered[serial_number], Device)

    def test_discover_network(self, test_data: TestData, mock_info_from_service: Mock):
        """Test discovering the network synchronously."""
        with patch("devolo_plc_api.network.ServiceBrowser", MockServiceBrowser), patch(
            "devolo_plc_api.network.Zeroconf.get_service_info", return_value=""
        ):
            serial_number = test_data.device_info[SERVICE_TYPE].properties["SN"]
            mock_info_from_service.return_value = ZeroconfServiceInfo(
                address=inet_aton(test_data.ip),
                properties=test_data.device_info[SERVICE_TYPE].properties,
            )
            discovered = network.discover_network(timeout=0.1)
            assert serial_number in discovered
            assert isinstance(discovered[serial_number], Device)

    def test_add_wrong_state(self):
        """Test early return on wrong state changes."""
        with patch("devolo_plc_api.network.Zeroconf.get_service_info") as gsi:
            network._add(Zeroconf(), SERVICE_TYPE, SERVICE_TYPE, ServiceStateChange.Removed, devices={}, timeout=3)
            assert gsi.call_count == 0

    def test_no_devices(self):
        """Test discovery with no devices."""
        with patch("devolo_plc_api.network.ServiceBrowser", MockServiceBrowser), patch(
            "devolo_plc_api.network.Zeroconf.get_service_info", return_value=None
        ):
            discovered = network.discover_network(timeout=0.1)
            assert not discovered

    @pytest.mark.parametrize("mt", ["2600", "2601"])
    def test_hcu(self, test_data: TestData, mt: str, mock_info_from_service: Mock):
        """Test ignoring Home Control Central Units."""
        with patch("devolo_plc_api.network.ServiceBrowser", MockServiceBrowser), patch(
            "devolo_plc_api.network.Zeroconf.get_service_info", return_value=""
        ):
            mock_info_from_service.return_value = ZeroconfServiceInfo(address=test_data.ip.encode(), properties={"MT": mt})
            discovered = network.discover_network(timeout=0.1)
            assert not discovered



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: multiap.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rmultiap.proto\x12\ndevice.api\"W\n\x16WifiMultiApGetResponse\x12\x0f\n\x07\x65nabled\x18\x01 \x01(\x08\x12\x15\n\rcontroller_id\x18\x02 \x01(\t\x12\x15\n\rcontroller_ip\x18\x03 \x01(\tB\x11\n\x06\x64\x65viceB\x07Multiapb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'multiap_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\006deviceB\007Multiap'
  _WIFIMULTIAPGETRESPONSE._serialized_start=29
  _WIFIMULTIAPGETRESPONSE._serialized_end=116
# @@protoc_insertion_point(module_scope)
